"""Websocket Request Messages."""
import json
from abc import ABC, abstractmethod

from weatherflow4py.models.ws.types import LightingStrikeType


class WebsocketRequest(ABC):
    def __init__(self, type: str, id: str):
        self.type = type
        self.id = id

    @abstractmethod
    def to_dict(self):
        pass

    @property
    def json(self) -> str:
        return json.dumps(self.to_dict())


class ListenStartMessage(WebsocketRequest):
    def __init__(self, device_id: str):
        super().__init__("listen_start", "2098388936")
        self.device_id = device_id

    def to_dict(self):
        return {"type": self.type, "device_id": self.device_id, "id": self.id}


class ListenStopMessage(WebsocketRequest):
    def __init__(self, device_id: str):
        super().__init__("listen_stop", "2098388936")
        self.device_id = device_id

    def to_dict(self):
        return {"type": self.type, "device_id": self.device_id, "id": self.id}


class GeoStrikeListenStartMessage(WebsocketRequest):
    def __init__(
        self,
        lat_min: float,
        lat_max: float,
        lon_min: float,
        lon_max: float,
        strike_type: LightingStrikeType = None,
    ):
        super().__init__("geo_strike_listen_start", "whatever")
        self.lat_min = lat_min
        self.lat_max = lat_max
        self.lon_min = lon_min
        self.lon_max = lon_max
        self.strike_type = strike_type

    def to_dict(self):
        message_dict = {
            "type": self.type,
            "lat_min": self.lat_min,
            "lat_max": self.lat_max,
            "lon_min": self.lon_min,
            "lon_max": self.lon_max,
            "id": self.id,
        }
        if self.strike_type is not None:
            message_dict["strike_type"] = self.strike_type
        return message_dict


class RapidWindListenStartMessage(WebsocketRequest):
    def __init__(self, device_id: str):
        super().__init__("listen_rapid_start", "2098388542")
        self.device_id = device_id

    def to_dict(self):
        return {"type": self.type, "device_id": self.device_id, "id": self.id}


class RapidWindListenStopMessage(WebsocketRequest):
    def __init__(self, device_id: str):
        super().__init__("listen_rapid_stop", "2098388587")
        self.device_id = device_id

    def to_dict(self):
        return {"type": self.type, "device_id": self.device_id, "id": self.id}



from dataclasses import dataclass
from typing import List, Union

from dataclasses_json import dataclass_json

from weatherflow4py.models.rest.status import Status
from weatherflow4py.models.ws.custom_types import (
    PrecipitationAnalysisType,
    ObservationType,
)
from weatherflow4py.models.ws.obs import (
    obs_st,
    WebsocketObservation,
    obs_sky,
    obs_air,
    ObservationFactory,
)


@dataclass_json
@dataclass
class Summary:
    pressure_trend: str
    strike_count_1h: int
    strike_count_3h: int
    precip_total_1h: float
    strike_last_dist: int
    strike_last_epoch: int
    precip_accum_local_yesterday: float
    precip_accum_local_yesterday_final: float
    precip_analysis_type_yesterday: PrecipitationAnalysisType
    feels_like: float
    heat_index: float
    wind_chill: float


@dataclass_json
@dataclass
class DeviceObservationREST:
    """Higher level class used by REST"""

    status: Status
    device_id: int
    type: ObservationType
    source: str
    summary: Summary
    obs: List[Union[obs_sky, obs_st, obs_air]]  # Add this line

    #
    def __post_init__(self):
        # Transform the raw observation data into the correct instances
        self.obs = ObservationFactory.create_observation(self.type, self.obs)


@dataclass_json
@dataclass
class DeviceObservationTempestREST(DeviceObservationREST, WebsocketObservation):
    """Wrap it up."""
